/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-new-project.h"

#include <gdk/gdkkeysyms.h>
#include <sys/wait.h>
#include <unistd.h>

static void openldev_new_project_dialog_populate_combobox (NewProjectDialog*);
static void openldev_new_project_dialog_on_next (GtkWidget*, NewProjectDialog*);
static void openldev_new_project_dialog_on_back (GtkWidget*, NewProjectDialog*);
static void openldev_new_project_dialog_on_apply  (GtkWidget*, NewProjectDialog*);
static void openldev_new_project_dialog_on_cancel (GtkWidget*, NewProjectDialog*);
static gboolean openldev_new_project_dialog_space (GtkWidget*, GdkEventKey*, gpointer);
string openldev_new_project_dialog_check_for_errors (NewProjectDialog*);

// Load the new project dialog, connect the signals and set the default project
// location to the user's home directory + /openldev/.
NewProjectDialog* 
openldev_new_project_dialog_new ()
{
  NewProjectDialog *project = new NewProjectDialog ();
  
  string gladefile = openldev_add_prefix ("/share/openldev/glade/newproject.glade");
  project->xml = glade_xml_new (gladefile.c_str(), NULL, NULL);

  project->dialog = glade_xml_get_widget   (project->xml, "dialog");
  project->combobox = glade_xml_get_widget (project->xml, "type");
  project->name = glade_xml_get_widget     (project->xml, "name");
  project->version = glade_xml_get_widget  (project->xml, "version");
  project->homepage = glade_xml_get_widget (project->xml, "homepage");
  project->email = glade_xml_get_widget    (project->xml, "email");
  project->author = glade_xml_get_widget   (project->xml, "author");
  project->location = glade_xml_get_widget (project->xml, "location");
  project->cancel = glade_xml_get_widget   (project->xml, "cancel");
  project->apply = glade_xml_get_widget    (project->xml, "apply");
  project->back = glade_xml_get_widget     (project->xml, "back");
  project->next = glade_xml_get_widget     (project->xml, "next");
  project->logo1 = glade_xml_get_widget    (project->xml, "logo1");
  project->logo2 = glade_xml_get_widget    (project->xml, "logo2");
  project->logo3 = glade_xml_get_widget    (project->xml, "logo3");
  project->notebook = glade_xml_get_widget (project->xml, "notebook1");
  project->confirm = glade_xml_get_widget  (project->xml, "confirm");
  
  openldev_new_project_dialog_populate_combobox (project);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (project->location),
                                       g_get_home_dir());
  
  gtk_image_set_from_file (GTK_IMAGE (project->logo1), (gchar*) openldev_add_prefix
                           ("/share/pixmaps/openldev/openldev24.png").c_str());
  gtk_image_set_from_file (GTK_IMAGE (project->logo2), (gchar*) openldev_add_prefix
                           ("/share/pixmaps/openldev/openldev24.png").c_str());
  gtk_image_set_from_file (GTK_IMAGE (project->logo3), (gchar*) openldev_add_prefix
                           ("/share/pixmaps/openldev/openldev24.png").c_str());
  
  gtk_widget_hide (project->apply);
  gtk_widget_set_sensitive (project->back, FALSE);
  
  // Prevent spaces from being entered into the entries
  g_signal_connect (G_OBJECT (project->name), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->version), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);
  
  g_signal_connect (G_OBJECT (project->next), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_next),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->back), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_back),
                    (gpointer) project);

  g_signal_connect (G_OBJECT (project->apply), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_apply),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->cancel), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_cancel), 
                    (gpointer) project);
  
  return project;
}

// Run the new project dialog and return the project file that was created
// if the project was created.
vector<string> 
openldev_new_project_dialog_run (NewProjectDialog* project)
{
  vector<string> list;
  string text, command;
  
  int result = gtk_dialog_run (GTK_DIALOG (project->dialog));
  
  if (result == GTK_RESPONSE_APPLY)
  {
    string type = gtk_combo_box_get_active_text (GTK_COMBO_BOX (project->combobox)), script;
    
    string directory = openldev_add_prefix ("/share/openldev/templates/"), comm, txt;
    unsigned int l;
    vector<string> TRUE_entries;
    vector<string> entries;
    const char* fn;
 
    GDir *dir = g_dir_open (directory.c_str(), 0, NULL);
    while ((fn = g_dir_read_name(dir)) != NULL)
      entries.push_back (fn);

    // Make sure they are actually plugins
    for (l=0; l < entries.size(); l++)
      if (entries[l].find(".sh", 0) != string::npos)
        TRUE_entries.push_back (entries[l]);
    entries.clear ();

    for (unsigned int i=0; i < TRUE_entries.size(); i++) 
    {
      gchar buff [8];
      comm = "sh " + directory + TRUE_entries[i] + " -n";
      txt = "\0";
      FILE *in = popen (comm.c_str(), "r");
      while (fgets (buff, sizeof (buff), in))
        txt += buff;
      pclose (in);
      
      if (type == txt)
      {
        script = comm.substr (3, comm.length() - 6);
        break;
      }
    }
    
    list.push_back (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (project->location)));
    list.push_back (gtk_entry_get_text (GTK_ENTRY (project->name)));
    list.push_back (gtk_entry_get_text (GTK_ENTRY (project->version)));
    list.push_back (gtk_entry_get_text (GTK_ENTRY (project->author)));
    list.push_back (gtk_entry_get_text (GTK_ENTRY (project->homepage)));
    list.push_back (gtk_entry_get_text (GTK_ENTRY (project->email)));
    list.push_back (script);
  }
  
  gtk_widget_destroy (project->dialog);
  g_object_unref (project->xml);
  return list;
}

// This adds the different types of projects to the combobox
static void 
openldev_new_project_dialog_populate_combobox (NewProjectDialog *project)
{ 
  string directory = openldev_add_prefix ("/share/openldev/templates/"), text, command;
  unsigned int l;
  vector<string> TRUE_entries;
  vector<string> entries;
  const char* fn;
  
  GDir *dir = g_dir_open (directory.c_str(), 0, NULL);
  while ((fn = g_dir_read_name(dir)) != NULL)
    entries.push_back (fn);
  
  // Make sure they are
  for (l=0; l < entries.size(); l++)
    if (entries[l].find(".sh", 0) != string::npos)
      TRUE_entries.push_back (entries[l]);
  entries.clear ();

  // Do the actual population of the types
	gtk_combo_box_remove_text (GTK_COMBO_BOX (project->combobox), 0);
	for (unsigned int i=0; i < TRUE_entries.size(); i++) 
  {
    gchar buff [8];
    command = "sh " + directory + TRUE_entries[i] + " -n";
    text = "\0";
    FILE *in = popen (command.c_str(), "r");
    while (fgets (buff, sizeof(buff), in))
      text += buff;
    pclose (in);
    
    gtk_combo_box_append_text (GTK_COMBO_BOX (project->combobox), (gchar*) text.c_str());
	}

  gtk_combo_box_set_active (GTK_COMBO_BOX (project->combobox), 0);
}

static void 
openldev_new_project_dialog_on_back (GtkWidget *widget, 
                                     NewProjectDialog *ptr)
{
  gint page = gtk_notebook_get_current_page (GTK_NOTEBOOK (ptr->notebook));
  
  if (page == 2)
  {
    gtk_widget_hide (ptr->apply);
    gtk_widget_show (ptr->next);
  }
  
  if (page > 0)
    gtk_notebook_set_current_page (GTK_NOTEBOOK (ptr->notebook), page - 1);
}

static void 
openldev_new_project_dialog_on_next (GtkWidget *widget, 
                                     NewProjectDialog *ptr)
{
  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (ptr->notebook)) == 0)
  {
    string errortext = openldev_new_project_dialog_check_for_errors (ptr);
  
    if (errortext == "NONE")
    {
      gtk_widget_set_sensitive (ptr->back, TRUE);
      gtk_notebook_set_current_page (GTK_NOTEBOOK (ptr->notebook), 1);
    }
    else
      openldev_error (errortext);
  }
  else
  {
    string type = gtk_combo_box_get_active_text (GTK_COMBO_BOX (ptr->combobox)),
           location = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (ptr->location)),
           name = gtk_entry_get_text (GTK_ENTRY (ptr->name)),
           version = gtk_entry_get_text (GTK_ENTRY (ptr->version)),
           author = gtk_entry_get_text (GTK_ENTRY (ptr->author)),
           homepage = gtk_entry_get_text (GTK_ENTRY (ptr->homepage)),
           email = gtk_entry_get_text (GTK_ENTRY (ptr->email));
    
    string text = "Confirm the following information:\n\nProject Type: " + type
                + "\nLocation: " + location + "\nName: " + name + "\nVersion: "
                + version + "\n\nAuthor: " + author + "\nE-mail: " + email
                + "\nHomepage: " + homepage;
  
    gtk_label_set_text (GTK_LABEL (ptr->confirm), text.c_str());
    gtk_widget_hide (ptr->next);
    gtk_widget_show (ptr->apply);
    
    gtk_notebook_set_current_page (GTK_NOTEBOOK (ptr->notebook), 2);
  }
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
static void 
openldev_new_project_dialog_on_cancel (GtkWidget *widget, 
                                       NewProjectDialog *ptr)
{
  gtk_dialog_response (GTK_DIALOG (ptr->dialog), GTK_RESPONSE_CANCEL);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
static void 
openldev_new_project_dialog_on_apply (GtkWidget *widget, 
                                      NewProjectDialog *ptr)
{
  string errortext = openldev_new_project_dialog_check_for_errors (ptr);
  
  // Create new project
  if (errortext == "NONE")
    gtk_dialog_response (GTK_DIALOG (ptr->dialog), GTK_RESPONSE_APPLY);
  // Throw an error
  else
    openldev_error (errortext);
}

string 
openldev_new_project_dialog_check_for_errors (NewProjectDialog* project)
{
  string ntxt = gtk_entry_get_text (GTK_ENTRY (project->name)),
         vtxt = gtk_entry_get_text (GTK_ENTRY (project->version)),
         ltxt = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (project->location));
  
  // Check for error when creating a project or file
  if (ntxt.empty())
    return "<span weight='bold'>OpenLDev Error:</span>\n\nYou must specify a project name!";
  else if (!isalpha (ntxt[0]))
    return "<span weight='bold'>OpenLDev Error:</span>\n\nThe first character of a project name must be a letter!";

  for (unsigned int i=1; i < ntxt.length(); i++)
    if (!isalnum (ntxt[i]) && ntxt[i] != '_')
      return "<span weight='bold'>OpenLDev Error:</span>\n\nThe project name must be alphanumeric!";
      
  if (vtxt.empty())
    return "<span weight='bold'>OpenLDev Error:</span>\n\nYou must specify a version number!";
  else if (ltxt.empty())
    return "<span weight='bold'>OpenLDev Error:</span>\n\nYou must specify a project location!";
  else
    return "NONE";
}

static gboolean 
openldev_new_project_dialog_space (GtkWidget *widget, 
                                   GdkEventKey *event,
                                   gpointer data)
{
  if (event->keyval == GDK_space ||
      event->keyval == GDK_KP_Space)
    return TRUE;
  
  return FALSE;
}
